@@ -49,6 +49,8 @@ struct _ExmBrowsePage
     int current_page;
     int max_pages;
 
+    GCancellable *cancellable;
+
     // Template Widgets
     GtkSearchEntry      *search_entry;
     GtkListBox          *search_results;
@@ -237,9 +239,13 @@ on_load_more_results (GtkButton     *btn,
     ExmSearchSort sort;
     gtk_widget_set_sensitive (GTK_WIDGET (self->more_results_btn), FALSE);
 
+    // If we have a current operation, cancel it
+    g_cancellable_cancel (self->cancellable);
+    self->cancellable = g_cancellable_new ();
+
     query = gtk_editable_get_text (GTK_EDITABLE (self->search_entry));
     sort = (ExmSearchSort) gtk_drop_down_get_selected (self->search_dropdown);
-    exm_search_provider_query_async (self->search, query, ++self->current_page, sort, NULL,
+    exm_search_provider_query_async (self->search, query, ++self->current_page, sort, self->cancellable,
                                      (GAsyncReadyCallback) on_next_page_result,
                                      self);
 }
@@ -256,7 +262,11 @@ search (ExmBrowsePage *self,
     if (self->search_results_model)
         g_clear_object (&self->search_results_model);
 
-    exm_search_provider_query_async (self->search, query, 1, sort, NULL,
+    // If we have a current operation, cancel it
+    g_cancellable_cancel (self->cancellable);
+    self->cancellable = g_cancellable_new ();
+
+    exm_search_provider_query_async (self->search, query, 1, sort, self->cancellable,
                                      (GAsyncReadyCallback) on_first_page_result,
                                      self);
 }
@@ -391,6 +401,9 @@ exm_browse_page_class_init (ExmBrowsePageClass *klass)
     gtk_widget_class_bind_template_child (widget_class, ExmBrowsePage, more_results_btn);
 
     gtk_widget_class_bind_template_callback (widget_class, on_search_entry_realize);
+    gtk_widget_class_bind_template_callback (widget_class, on_search_changed);
+    gtk_widget_class_bind_template_callback (widget_class, on_load_more_results);
+    gtk_widget_class_bind_template_callback (widget_class, on_bind_manager);
 
     gtk_widget_class_set_layout_manager_type (widget_class, GTK_TYPE_BIN_LAYOUT);
 }
@@ -417,30 +430,6 @@ exm_browse_page_init (ExmBrowsePage *self)
 
     g_object_unref (settings);
 
-    g_signal_connect_swapped (self->search_entry,
-                              "search-changed",
-                              G_CALLBACK (on_search_changed),
-                              self);
-
-    g_signal_connect_swapped (self->search_dropdown,
-                              "notify::selected",
-                              G_CALLBACK (on_search_changed),
-                              self);
-
-    g_signal_connect (self->search_entry,
-                      "realize",
-                      G_CALLBACK (on_search_entry_realize),
-                      self);
-
-    g_signal_connect (self->more_results_btn,
-                      "clicked",
-                      G_CALLBACK (on_load_more_results),
-                      self);
-
-    g_signal_connect (self,
-                      "notify::manager",
-                      G_CALLBACK (on_bind_manager),
-                      NULL);
-
     load_suggestions (self);
 }
+
