@@ -3,6 +3,7 @@
 #include <adwaita.h>
 
 #include "local/exm-extension.h"
+#include "local/exm-manager.h"
 
 G_BEGIN_DECLS
 
@@ -11,6 +12,7 @@ G_BEGIN_DECLS
 G_DECLARE_FINAL_TYPE (ExmExtensionRow, exm_extension_row, EXM, EXTENSION_ROW, AdwExpanderRow)
 
 ExmExtensionRow *
-exm_extension_row_new (ExmExtension *extension);
+exm_extension_row_new (ExmExtension *extension,
+                       ExmManager   *manager);
 
 G_END_DECLS
