@@ -11,14 +11,14 @@ typedef enum
 
 typedef enum
 {
-    EXM_EXTENSION_STATE_ENABLED = 1,
-    EXM_EXTENSION_STATE_DISABLED = 2,
+    EXM_EXTENSION_STATE_ACTIVE = 1,
+    EXM_EXTENSION_STATE_INACTIVE = 2,
     EXM_EXTENSION_STATE_ERROR = 3,
     EXM_EXTENSION_STATE_OUT_OF_DATE = 4,
     EXM_EXTENSION_STATE_DOWNLOADING = 5,
     EXM_EXTENSION_STATE_INITIALIZED = 6,
-    EXM_EXTENSION_STATE_DISABLING = 7,
-    EXM_EXTENSION_STATE_ENABLING = 8,
+    EXM_EXTENSION_STATE_DEACTIVATING = 7,
+    EXM_EXTENSION_STATE_ACTIVATING = 8,
     EXM_EXTENSION_STATE_UNINSTALLED = 99,
 } ExmExtensionState;
 
